local Sackville = LibStub("AceAddon-3.0"):GetAddon("Sackville", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Sackville")

Sackville.Config = {}
Sackville.Config.__index = Sackville.Config

local AceGUI = LibStub("AceGUI-3.0")
local _

function Sackville.Config:CreateGUI()   
    local sframe = AceGUI:Create("Frame")
    sframe:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
    sframe:SetTitle("Sackville Config Test")
    sframe:SetLayout("Fill")
    
    stree = AceGUI:Create("TreeGroup")
    stree:SetTree(MakeTree())
    stree:SetLayout("Fill")
    sframe:AddChild(stree)
    stree:SetCallback("OnGroupSelected", TreeNodeSelected)
    
    Sackville.configFrame = sframe
    Sackville.configFrame.treeGroup = stree
end

function TreeNodeSelected(treeGroup, event, pathString)
    treeGroup:ReleaseChildren()
    local group, elements = strsplit("\001", pathString, 2)
    
    if group == L["Bags"] then
        ShowBagSectionGroup(treeGroup, elements)
    elseif group == CATEGORIES then
        -- Sackville:Print("Clicked in Categories group")
    end
        
end

function ShowBagSectionGroup(treeGroup, pathString)
    if pathString then
        local bagName, sectionName = strsplit("\001", pathString)

        local bag, section
        if bagName then
            bag = Sackville:GetBag(bagName)
            if bag and sectionName then
                section = bag.sections[sectionName]
            end
        end
        
        local bagOpts, sectionOpts
        if bagName then
            bagOpts = Sackville.db.profile.bagConfig[bagName]
            if sectionName then
                sectionOpts = bagOpts.sections[sectionName]
            end
        end
            
        if sectionOpts then -- configure this particular section
            ShowSectionOptions(treeGroup, sectionOpts)
        elseif bagOpts then -- configure this particular bag
            ShowBagOptions(treeGroup, bagOpts)
        end
    else -- Show general 
        ShowBagList(treeGroup)
    end
end

function ShowSectionOptions(treeGroup, sectionOpts)
    local sectionGroup = AceGUI:Create("SimpleGroup")
    sectionGroup:SetLayout("Flow")
    
    local editName = AceGUI:Create("EditBox")
    editName:SetLabel(L["Section name"])
    editName:SetText(sectionOpts.name)
    sectionGroup:AddChild(editName)
       
    local cbIsDefault = AceGUI:Create("CheckBox")
    cbIsDefault:SetLabel(L["Match all items"])
    cbIsDefault:SetValue(sectionOpts.alwaysMatch)
    sectionGroup:AddChild(cbIsDefault)
    
    local slidePriority = AceGUI:Create("Slider")
    slidePriority:SetSliderValues(0, 10, 1)
    slidePriority:SetValue(sectionOpts.priority)
    slidePriority:SetLabel(L["Priority"])
    slidePriority:SetDisabled(sectionOpts.useDefaultPriority)
    sectionGroup:AddChild(slidePriority)
    
    
    local cbUseDefaultPriority = AceGUI:Create("CheckBox")
    cbUseDefaultPriority:SetLabel(L["Use default priority"])
    cbUseDefaultPriority:SetValue(sectionOpts.useDefaultPriority)
    sectionGroup:AddChild(cbUseDefaultPriority)
    
    treeGroup:AddChild(sectionGroup)
end

function ShowBagOptions(treeGroup, bagOpts)
    local sfContents = AceGUI:Create("ScrollFrame")
    sfContents:SetLayout("Flow")
    
    local editName = AceGUI:Create("EditBox")
    editName:SetLabel(L["Bag name"])
    editName:SetText(bagOpts.name)
    sfContents:AddChild(editName)
    
    local ddType = AceGUI:Create("Dropdown")
    ddType:SetLabel(L["Type"])
    ddType:SetList( Sackville.Options.bagTypeList ) 
    -- ddType:AddItem("player", "player")
    -- ddType:AddItem("bank", "bank")
    ddType:SetValue(bagOpts.bagType)
    ddType:SetWidth(90)
    sfContents:AddChild(ddType)

    local slideDefaultPriority = AceGUI:Create("Slider")
    slideDefaultPriority:SetSliderValues(0, 10, 1)
    slideDefaultPriority:SetValue(bagOpts.defaultPriority)
    slideDefaultPriority:SetLabel(L["Default section priority"])
    sfContents:AddChild(slideDefaultPriority)
    
    local igSectionList = AceGUI:Create("InlineGroup")
    igSectionList:SetTitle(L["Sections"])
    igSectionList:SetRelativeWidth(1)
    
    for _, sectionOpts in SectionOptIterator(bagOpts) do
        local sectionGroup = AceGUI:Create("SimpleGroup")
        sectionGroup:SetLayout("Flow")
        sectionGroup:SetRelativeWidth(1)
        igSectionList:AddChild(sectionGroup)
        
        local editSection = AceGUI:Create("EditBox")
        editSection:SetText(sectionOpts.name)
        sectionGroup:AddChild(editSection)
        
        local iconUp = AceGUI:Create("SackvilleIcon")
        iconUp:SetImage("INTERFACE/CHATFRAME/UI-ChatIcon-ScrollUp-Up")
        iconUp:SetImageSize(28,28)
        iconUp:SetWidth(28)
        sectionGroup:AddChild(iconUp)
        
        local iconDown = AceGUI:Create("SackvilleIcon")
        iconDown:SetImage("INTERFACE/CHATFRAME/UI-ChatIcon-ScrollDown-Up")
        iconDown:SetImageSize(28,28)
        iconDown:SetWidth(28)
        sectionGroup:AddChild(iconDown)    

        local btnRemove = AceGUI:Create("SackvilleButton")
        btnRemove:SetAutoWidth(true)
        btnRemove:SetText(REMOVE)
        sectionGroup:AddChild(btnRemove)        
    end
    sfContents:AddChild(igSectionList)
    treeGroup:AddChild(sfContents)
end

function MakeBagSortOptions()

end


function ShowBagList(treeGroup)
    local sfContents = AceGUI:Create("ScrollFrame")
    sfContents:SetLayout("List")
    treeGroup:AddChild(sfContents)
    
    -- Add controls to create a new bag
    local sgNewBagControls = AceGUI:Create("SimpleGroup")
    sgNewBagControls:SetLayout("Flow")        
    
    local editNewBag = AceGUI:Create("EditBox")
    editNewBag:SetLabel(L["New bag name"])
    sgNewBagControls:AddChild(editNewBag)        
    
    local btnCreateBag = AceGUI:Create("SackvilleButton")
    btnCreateBag:SetText(CALENDAR_CREATE)
    btnCreateBag:SetAutoWidth(true)
    sgNewBagControls:AddChild(btnCreateBag)        
    
    sfContents:AddChild(sgNewBagControls)
    
    -- Add a list of all current bags with buttons to reorder/delete
    local igBagList = AceGUI:Create("InlineGroup")
    igBagList:SetTitle(L["Bags"])
    igBagList:SetRelativeWidth(1)
    sfContents:AddChild(igBagList)
    
    for _, bagOpts in BagOptIterator() do
        local bagGroup = AceGUI:Create("SimpleGroup")
        bagGroup:SetLayout("Flow")
        bagGroup:SetRelativeWidth(1)
        igBagList:AddChild(bagGroup)
        
        local editBagName = AceGUI:Create("EditBox")
        editBagName:SetText(bagOpts.name)
        bagGroup:AddChild(editBagName)
               
        local iconUp = AceGUI:Create("SackvilleIcon")
        iconUp:SetImage("INTERFACE/CHATFRAME/UI-ChatIcon-ScrollUp-Up")
        iconUp:SetImageSize(28,28)
        iconUp:SetWidth(28)
        bagGroup:AddChild(iconUp)
        
        local iconDown = AceGUI:Create("SackvilleIcon")
        iconDown:SetImage("INTERFACE/CHATFRAME/UI-ChatIcon-ScrollDown-Up")
        iconDown:SetImageSize(28,28)
        iconDown:SetWidth(28)
        bagGroup:AddChild(iconDown)
        
        local btnRemove = AceGUI:Create("SackvilleButton")
        btnRemove:SetAutoWidth(true)
        btnRemove:SetText(REMOVE)
        bagGroup:AddChild(btnRemove)
    end   
end

function MakeTree()
    local tree = {}
    local bags = {
        text = L["Bags"] .. " / " .. L["Sections"],
        value = L["Bags"],
        children = { }
    }
       
    for _, bagOpts in BagOptIterator() do
        local bagNode = { text = bagOpts.name, value = bagOpts.name, children = {} }
        
        for _, sectionOpts in SectionOptIterator(bagOpts) do            
            local sectionNode = { text = sectionOpts.name, value = sectionOpts.name }
            table.insert(bagNode.children, sectionNode)
        end
        table.insert(bags.children, bagNode)
    end
    
    local categories = {
        text = CATEGORIES,
        value = CATEGORIES,
        children = { }
    }
    
    local catList = {}
    for catName, _ in pairs(Sackville.db.profile.categories) do
        table.insert(catList, catName)
    end
    table.sort(catList)
    for _, catName in ipairs(catList) do
        table.insert(categories.children, { text = catName, value = catName })
    end
    
    table.insert(tree, bags)
    table.insert(tree, categories)
    return tree
end

function BagOptIterator()
    local bagSortOptions = { {key = "displayOrder", comparison = "<" } }    
    return Sackville.Util.kpairs(Sackville.db.profile.bagConfig, bagSortOptions)
end

function SectionOptIterator(bagOpts)
    sectionSortCritera = { { key = "displayOrder", comparison = "<" } }
    return Sackville.Util.kpairs(bagOpts.sections, sectionSortCritera)
end