local Sackville = LibStub("AceAddon-3.0"):GetAddon("Sackville", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Sackville")
Sackville.Filter = {}
local allFilters = {}
Sackville.allFilters = allFilters

local function AlwaysFalse() return false end

function Sackville.Filter:AddFilter(path, filter, tree)
-- path: string with \001 separators for each level of depth
-- filter: A function taking an Item which returns true/false indicating if
-- the item is a match.
    if tree == nil then
        local debugPath = string.gsub(path, "\001", "/")
        Sackville:Debug(1, "filter", "AddFilter path=%s", debugPath)
        tree = allFilters
    end
    
    local current, rest = strsplit("\001", path, 2)
    if not tree[current] then 
        tree[current] = { filter = AlwaysFalse } 
    end
    if rest then 
        Sackville.Filter:AddFilter(rest, filter, tree[current])
    else 
        tree[current].filter = filter
    end
    
end

function Sackville.Filter:GetFilter(path, tree)
    tree = tree or allFilters
    local debugPath = string.gsub(path, "\001", "/")
    local current, rest = strsplit("\001", path, 2)
    Sackville:Debug(3, "filter", "GetFilter(%s)", debugPath)
    assert(tree[current], "Tried loading non-existant filter with path: " .. path)
    if rest then
        return Sackville.Filter:GetFilter(rest, tree[current])
    else
        return tree[current].filter
    end
end

function Sackville.Filter:ItemMatchesRules(item, matchRules, ruleIndex)
    ruleIndex = ruleIndex or 1
    
    local rule = matchRules[ruleIndex]
    local returnValue = false

    if rule then
        rule.operation = rule.operation or "and"    -- default
        rule.negate = rule.negate or false          -- default
        assert(rule.operation == "and" or rule.operation == "or")
        local filter = Sackville.Filter:GetFilter(rule.filterPath)
        returnValue = filter(item)
        if rule.negate then returnValue = not returnValue end
    end
    
    if matchRules[ruleIndex+1] and rule.operation == "and" then
        returnValue = returnValue and 
            Sackville.Filter:ItemMatchesRules(item, matchRules, ruleIndex+1)
    elseif matchRules[ruleIndex+1] and rule.operation == "or" then
        returnValue = returnValue or
            Sackville.Filter:ItemMatchesRules(item, matchRules, ruleIndex+1)    
    end

    return returnValue
end

